#ifndef _INCLUDE_PARSE_H
#define _INCLUDE_PARSE_H

#include <sys/time.h>

#define ACTION_TIMER    0
#define ACTION_EVENT    1
#define ACTION_PARSE    2

#define EVENT_RANGE     0
#define EVENT_NEWNODE   1
#define EVENT_LOCALIZE  2

#define METHOD_ROBUST_QUADS 0
#define METHOD_MOBILE_LSQ   1

#define POS_FLAG_WEAK   1

typedef struct _pos {
    int             node;
    float           x, y;
    int             flags;
} pos;

typedef struct _quad {
    unsigned char   v[3];
} quad;

typedef struct _event {
    int             type;
    struct timeval  time;
    struct _event  *next;
} event;

typedef struct _event_range {
    struct _event   evt;
    int             node_recv;
    int             node_send;
    int             summary;
    int             dist;
} event_range;

typedef struct _event_newnode {
    struct _event   evt;
    int             node;
    int             self;
    unsigned char   id[4];
} event_newnode;

typedef struct _event_localize {
    struct _event   evt;
    int             method;
    int             num_pos;
    struct _pos    *pos;
    int             num_opt;
    struct _pos    *opt;
    int             num_quad;
    struct _quad   *quad;
} event_localize;

typedef struct _event_queue {
    struct _event  *head;
    struct _event  *tail;
} event_queue;


struct _parse_info;

typedef int (*cb_range_t)(struct _event_range *, struct timeval *);
typedef int (*cb_newnode_t)(struct _event_newnode *, struct timeval *);
typedef int (*cb_localize_t)(struct _event_localize *, struct timeval *);
typedef int (*cb_timer_t)(struct timeval *);
typedef void (*parse_line_t)(struct _parse_info * parse, event_queue * events,
        char * str, long long linetime);


#define STATE_POS   0
#define STATE_OPT   1

typedef struct _parse_info {
    int             use_serial;
    int             serfd;
    FILE *          infile;
    struct timeval  starttime;
    int             realtime;
    int             readahead_time;
    long long       next_line_time;
    long long       start_line_time;
    int             has_lines;
    char            linebuf[256];
    FILE *          verbose;
    cb_range_t      cb_range;
    cb_newnode_t    cb_newnode;
    cb_localize_t   cb_localize;
    cb_timer_t      cb_timer;
    parse_line_t    line;
    event          *event_buf;
    int             state;
    int             selfnode;
    int             last_node;
    char            v1_nodes[256];
    int             hastime;
} parse_info;

extern parse_info * parser_init_serial(int fd, FILE * verbose);
extern parse_info * parser_init_file(FILE * infile, int realtime,
        int readahead_time, FILE * verbose);
extern void parser_free(parse_info * parse);
extern int parser_mainloop(parse_info * parse);

#endif /* _INCLUDE_PARSE_H */
