function [err,success] = plot_full_localization(name, nodes, radius)

sup = [];
if length(nodes) > 1
    sup = nodes(2:end);
end
i = nodes(1);

net = load([name '.mat']);
N = size(net.xy,1);

load([name '_tx.mat']);

if isempty(cluster_xy{i})
    fprintf('Empty cluster\n');
    return;
end

if ~isempty(sup)
    [tf, si] = ismember(sup, cluster_xy{i}(:,1));
else
    si = 1:size(cluster_xy{i},1);
end

opts = optimset('Display', 'on', 'MaxFunEvals', 400);
tx = [net.xy(i,:) 0];
[txnf, rnf] = lsqnonlin(@lsqfun, tx, [], [], opts, ...
    [[0;0] cluster_xy{i}(si,2:3)'], net.xy([i cluster_xy{i}(si,1)'],:)', 1);
[txf, rf]   = lsqnonlin(@lsqfun, tx, [], [], opts, ...
    [[0;0] cluster_xy{i}(si,2:3)'], net.xy([i cluster_xy{i}(si,1)'],:)', -1);

if rf < rnf
    flip = -1;
    tx = txf;
else
    flip = 1;
    tx = txnf;
end

new_xy = zeros(N,3);
for j=1:N,
    if routing(i,j) == 0
        continue
    end
    T = eye(3);
    num_t = 1;
    k = j;
    while (k ~= i)
        T = cluster_tx{routing(i,k), k} * T;
        num_t = num_t + 1;
        k = routing(i,k);
    end
    T = tx_to_matrix(tx, flip) * T;
    for m = 1:size(cluster_xy{j},1),
        k = cluster_xy{j}(m,1);
        p = [cluster_xy{j}(m,2:3)'; 1];
        if (new_xy(k,3) == 0 || num_t < new_xy(k,3))
            p = T * p;
            new_xy(k,1:2) = p(1:2)';
            new_xy(k,3) = num_t;
        end
    end
end

h = figure;
%plot(net.xy(:,1), net.xy(:,2), '+');
hold on;

idx = find(new_xy(:,3) > 0);
%p_new_xy = new_xy(idx,:);
%plot(p_new_xy(:,1), p_new_xy(:,2), 'ko');
p3 = plot(new_xy(nodes,1), new_xy(nodes,2), 'ko', 'MarkerSize', 9);
err = zeros(0,2);
for j = idx',
    p1 = plot(new_xy(j,1), new_xy(j,2), 'ko', 'MarkerSize', 6, 'LineWidth', 1, 'MarkerFaceColor', [0.7 0.7 1.0]);
    plot([new_xy(j,1) net.xy(j,1)], [new_xy(j,2) net.xy(j,2)], 'k-', 'LineWidth', 2);
    err = [err; new_xy(j,1)-net.xy(j,1) new_xy(j,2)-net.xy(j,2)];
end

idx = find(new_xy(:,3) == 0);
p2 = plot(net.xy(idx,1), net.xy(idx,2), 'rx', 'MarkerSize', 7, 'LineWidth', 1.5);

if isempty(sup)
    for j=cluster_xy{i}(:,1)',
        text(new_xy(j,1),new_xy(j,2),num2str(j));
    end
end

if nargin > 2
    plot(new_xy(i,1)+radius*cos([0:0.05:2*pi]), ...
        new_xy(i,2)+radius*sin([0:0.05:2*pi]), 'k--');
end

success = zeros(0,2);
for j=1:N,
    success = [success; size(cluster_xy{j},1)+1 sum(net.dist(j,:) ~= 0)+1];
end

axis equal;
axis([0 1500 0 1500]);

l = legend([p1 p2 p3], 'Localized node position', ...
    'Unlocalizable node', 'Anchor node');
%set(l, 'Position', [0.40 0.9 0.33 0.087]);

exportfig(h, [name '.eps'], 'Format', 'eps', 'width', 6, ...
    'height', 6, 'FontSize', 1, 'Resolution', 300, 'Color', 'cmyk');
exportfig(h, [name '.png'], 'Format', 'png', 'width', 6, ...
    'height', 6, 'FontSize', 1, 'Resolution', 150, 'Color', 'cmyk');

function e = lsqfun(tx, a, b, flip);
a(1,:) = flip*a(1,:);
a = transform(tx, a);
e = a-b;
e = [e(1,:) e(2,:)]';

function b = transform(tx, a);
b = [cos(tx(3)) -sin(tx(3)); sin(tx(3)) cos(tx(3))] * a;
b = [b(1,:)+tx(1); b(2,:)+tx(2)];

function t = tx_to_matrix(tx, flip);
t = [flip*cos(tx(3)) -sin(tx(3)) tx(1); flip*sin(tx(3)) cos(tx(3)) tx(2); 0 0 1];
